"""
Main Brain class for Ghost AI.
"""

import os
import datetime
import asyncio
from typing import List, Dict, Any, Optional, Tuple

from .config import BrainConfig
from .memory import MemoryManager
from .memory_txtai import TxtaiMemoryManager
from .llm import LLMHandler

import zipfile
import tempfile
import json


class Brain:
    """Main brain class that orchestrates memory and LLM functionality."""
    
    def __init__(self, config: Optional[BrainConfig] = None):
        self.config = config or BrainConfig()
        
        # Choose memory backend based on configuration
        if self.config.memory.backend == "txtai":
            try:
                self.memory = TxtaiMemoryManager(self.config)
                print(f"Using txtai memory backend")
            except ImportError:
                print("txtai not available, falling back to sentence-transformers backend")
                self.config.memory.backend = "sentence_transformers"
                self.memory = MemoryManager(self.config)
        else:
            self.memory = MemoryManager(self.config)
            
        self.llm = LLMHandler(self.config)
        self._initialized = False
        
    async def initialize(self):
        """Initialize the brain components."""
        if self._initialized:
            return
            
        print("Initializing Ghost Brain...")
        
        # Initialize memory database
        self.memory.initialize_database()
        
        # Test LLM connectivity
        await self._test_llm_connectivity()
        
        self._initialized = True
        print("Ghost Brain initialized successfully!")
        
    async def _test_llm_connectivity(self):
        """Test LLM connectivity."""
        print("Testing LLM connectivity...")
        
        if self.config.llm.openai_api_key:
            print("✓ OpenAI API key configured")
        if self.config.llm.gemini_api_key:
            print("✓ Gemini API key configured")
            
        if not self.config.llm.openai_api_key and not self.config.llm.gemini_api_key:
            print("⚠ No LLM API keys configured")
            
    async def process_message(
        self, 
        message: str, 
        history: Optional[List[Dict[str, Any]]] = None,
        user_id: Optional[str] = None,
        system_prompt: Optional[str] = None,
        model_type: Optional[str] = None,
        model_name: Optional[str] = None,
        use_structured_response: bool = True
    ) -> Dict[str, Any]:
        """Process a message and return response with memory integration."""
        
        if not self._initialized:
            await self.initialize()
            
        # Update user_id if provided
        if user_id:
            self.config.user_id = user_id
            
        # Use provided system prompt or default
        system_prompt = system_prompt or self.config.system_prompt
        
        # Get chat history with memory integration
        memory_history = await self.memory.get_chat_history(message)
        
        # Combine provided history with memory
        full_history = []
        if history:
            full_history.extend(history)
        full_history.extend(memory_history)
        
        # Create user prompt
        user_prompt = f"User: {message}"
        
        # Get LLM response
        if use_structured_response:
            structured_prompt = self.llm.create_structured_prompt(system_prompt)
            raw_response = await self.llm.get_response(
                system_prompt=structured_prompt,
                history=full_history,
                user_prompt=user_prompt,
                model_type=model_type,
                model_name=model_name
            )
            
            # Parse structured response
            response_to_user, reasoning = self.llm.parse_structured_response(raw_response or "")
        else:
            raw_response = await self.llm.get_response(
                system_prompt=system_prompt,
                history=full_history,
                user_prompt=user_prompt,
                model_type=model_type,
                model_name=model_name
            )
            response_to_user = raw_response or ""
            reasoning = None
            
        # Store in memory (background task)
        if history:
            asyncio.create_task(
                self.memory.process_and_store_memory(
                    history=history + [{"role": "user", "content": message, "message_id": "temp", "timestamp": datetime.datetime.now().isoformat(), "author_id": user_id or "user"}],
                    response=response_to_user,
                    reasoning=reasoning or "No reasoning provided"
                )
            )
            
        return {
            "response": response_to_user,
            "reasoning": reasoning,
            "memory_chunks": len(memory_history),
            "timestamp": datetime.datetime.now().isoformat(),
            "model_type": model_type or "auto",
            "model_name": model_name or "auto",
            "memory_backend": self.config.memory.backend
        }
        
    async def search_memory(self, query: str, top_k: int = 3) -> List[str]:
        """Search memory for relevant chunks."""
        if not self._initialized:
            await self.initialize()
            
        return await self.memory.search_similar_chunks(query, top_k)
        
    async def get_memory_stats(self) -> Dict[str, Any]:
        """Get memory statistics."""
        if not self._initialized:
            await self.initialize()
            
        # Get basic stats from memory manager
        stats = await self.memory.get_memory_stats()
        
        # Add brain-level stats
        stats.update({
            "initialized": self._initialized,
            "memory_backend": self.config.memory.backend
        })
        
        return stats
        
    async def embed_chat_history(self, chat_history_path: str, user_id: Optional[str] = None) -> Dict[str, Any]:
        """Embed chat history from nexus-ai-chat-importer format."""
        if not self._initialized:
            await self.initialize()
            
        import os
        import re
        from pathlib import Path
        
        files_processed = 0
        chunks_embedded = 0
        chunks_skipped = 0
        errors = []
        
        try:
            # Convert to Path object
            history_path = Path(chat_history_path)
            
            if not history_path.exists():
                return {
                    "status": "error",
                    "files_processed": 0,
                    "chunks_embedded": 0,
                    "chunks_skipped": 0,
                    "errors": [f"Chat history path does not exist: {chat_history_path}"]
                }
            
            # Walk through the directory structure
            for file_path in history_path.rglob("*.md"):
                try:
                    # Skip Reports directory
                    if "Reports" in str(file_path):
                        continue
                        
                    # Read the file content
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                    
                    # Parse the frontmatter and content
                    frontmatter_match = re.match(r'^---\s*\n(.*?)\n---\s*\n(.*)$', content, re.DOTALL)
                    if not frontmatter_match:
                        continue
                        
                    frontmatter_text, chat_content = frontmatter_match.groups()
                    
                    # Extract metadata from frontmatter
                    metadata = {}
                    for line in frontmatter_text.split('\n'):
                        if ':' in line:
                            key, value = line.split(':', 1)
                            metadata[key.strip()] = value.strip()
                    
                    # Extract conversations from content
                    conversations = []
                    current_conversation = []
                    
                    # Split by conversation markers
                    conversation_parts = re.split(r'### User, on .*?;\s*', chat_content)
                    
                    for i, part in enumerate(conversation_parts[1:], 1):  # Skip first empty part
                        # Extract user message
                        user_match = re.match(r'^> (.*?)(?=\n\n#### |$)', part, re.DOTALL)
                        if user_match:
                            user_message = user_match.group(1).strip()
                            
                            # Extract assistant response
                            assistant_match = re.search(r'#### .*?;\s*\n>> (.*?)(?=\n\n---|\n\n### |$)', part, re.DOTALL)
                            if assistant_match:
                                assistant_message = assistant_match.group(1).strip()
                                
                                # Extract UIDs from the messages
                                user_uid_match = re.search(r'<!-- UID: ([a-f0-9-]+) -->', user_message)
                                assistant_uid_match = re.search(r'<!-- UID: ([a-f0-9-]+) -->', assistant_message)
                                
                                user_uid = f"chat_{user_uid_match.group(1)}" if user_uid_match else f"chat_{file_path.stem}_user_{i}"
                                assistant_uid = f"chat_{assistant_uid_match.group(1)}" if assistant_uid_match else f"chat_{file_path.stem}_assistant_{i}"
                                
                                # Clean messages by removing UID comments
                                user_message_clean = re.sub(r'<!-- UID: [a-f0-9-]+ -->', '', user_message).strip()
                                assistant_message_clean = re.sub(r'<!-- UID: [a-f0-9-]+ -->', '', assistant_message).strip()
                                
                                # Check if messages already exist in database
                                user_exists = await self.memory.message_exists(user_uid)
                                assistant_exists = await self.memory.message_exists(assistant_uid)
                                
                                if user_exists and assistant_exists:
                                    chunks_skipped += 2
                                    continue
                                
                                # Add to conversation
                                current_conversation.extend([
                                    {
                                        "role": "user",
                                        "content": user_message_clean,
                                        "message_id": user_uid,
                                        "timestamp": metadata.get('create_time', ''),
                                        "author_id": user_id or "user"
                                    },
                                    {
                                        "role": "assistant", 
                                        "content": assistant_message_clean,
                                        "message_id": assistant_uid,
                                        "timestamp": metadata.get('create_time', ''),
                                        "author_id": "assistant"
                                    }
                                ])
                    
                    # Store conversation in memory
                    if current_conversation:
                        await self.memory.process_and_store_memory(
                            history=current_conversation,
                            response="",  # No response needed for historical data
                            reasoning=f"Historical chat from {file_path.name}"
                        )
                        chunks_embedded += len(current_conversation)
                    
                    files_processed += 1
                    
                except Exception as e:
                    error_msg = f"Error processing {file_path}: {str(e)}"
                    errors.append(error_msg)
                    print(f"Warning: {error_msg}")
            
            return {
                "status": "success",
                "files_processed": files_processed,
                "chunks_embedded": chunks_embedded,
                "chunks_skipped": chunks_skipped,
                "errors": errors
            }
            
        except Exception as e:
            return {
                "status": "error",
                "files_processed": files_processed,
                "chunks_embedded": chunks_embedded,
                "chunks_skipped": chunks_skipped,
                "errors": [f"General error: {str(e)}"] + errors
            }
        
    async def import_chatgpt_zip(self, file, user_id: Optional[str] = None) -> Dict[str, Any]:
        """Import ChatGPT export ZIP and embed conversations with attachments."""
        if not self._initialized:
            await self.initialize()
        import os
        import hashlib
        import mimetypes
        files_processed = 0
        chunks_embedded = 0
        chunks_skipped = 0
        attachments_processed = 0
        errors = []
        temp_dir = tempfile.mkdtemp()
        try:
            # Save uploaded ZIP to temp file
            zip_path = os.path.join(temp_dir, "chatgpt_export.zip")
            with open(zip_path, "wb") as f:
                f.write(await file.read())
            # Extract ZIP
            with zipfile.ZipFile(zip_path, "r") as zip_ref:
                zip_ref.extractall(temp_dir)
            # Parse conversations.json
            conv_path = os.path.join(temp_dir, "conversations.json")
            if not os.path.exists(conv_path):
                return {"status": "error", "files_processed": 0, "chunks_embedded": 0, "chunks_skipped": 0, "attachments_processed": 0, "errors": ["conversations.json not found in ZIP"]}
            with open(conv_path, "r", encoding="utf-8") as f:
                conversations = json.load(f)
            for conv in conversations:
                try:
                    # Each conversation has a mapping of messages
                    mapping = conv.get("mapping", {})
                    # Sort messages by create_time
                    messages = [m for m in mapping.values() if m.get("message")]
                    messages.sort(key=lambda m: m["message"].get("create_time", 0))
                    # Extract user/assistant pairs
                    current_conversation = []
                    for m in messages:
                        msg = m["message"]
                        role = msg.get("author", {}).get("role")
                        content = msg.get("content", {}).get("parts", [""])[0]
                        msg_id = msg.get("id")
                        timestamp = msg.get("create_time", "")
                        if role in ("user", "assistant") and content:
                            # Prefix UID
                            prefixed_id = f"chat_{msg_id}"
                            # Deduplication
                            exists = await self.memory.message_exists(prefixed_id)
                            if exists:
                                chunks_skipped += 1
                                continue
                            
                            # Process attachments for this message
                            message_attachments = []
                            if role == "user" and "attachments" in msg.get("content", {}):
                                attachments = msg["content"]["attachments"]
                                for attachment in attachments:
                                    try:
                                        attachment_info = await self._process_attachment(
                                            attachment, temp_dir, prefixed_id
                                        )
                                        if attachment_info:
                                            message_attachments.append(attachment_info)
                                            attachments_processed += 1
                                    except Exception as e:
                                        errors.append(f"Error processing attachment in message {msg_id}: {str(e)}")
                            
                            # Store message
                            current_conversation.append({
                                "role": role,
                                "content": content,
                                "message_id": prefixed_id,
                                "timestamp": timestamp,
                                "author_id": user_id or role,
                                "attachments": message_attachments
                            })
                    if current_conversation:
                        await self.memory.process_and_store_memory(
                            history=current_conversation,
                            response="",
                            reasoning=f"Imported from ChatGPT ZIP: {conv.get('title', '')}"
                        )
                        chunks_embedded += len(current_conversation)
                    files_processed += 1
                except Exception as e:
                    errors.append(f"Error processing conversation: {str(e)}")
            return {"status": "success", "files_processed": files_processed, "chunks_embedded": chunks_embedded, "chunks_skipped": chunks_skipped, "attachments_processed": attachments_processed, "errors": errors}
        except Exception as e:
            return {"status": "error", "files_processed": files_processed, "chunks_embedded": chunks_embedded, "chunks_skipped": chunks_skipped, "attachments_processed": attachments_processed, "errors": [str(e)] + errors}
        finally:
            try:
                import shutil
                shutil.rmtree(temp_dir)
            except Exception:
                pass

    async def _process_attachment(self, attachment: Dict[str, Any], temp_dir: str, message_id: str) -> Optional[Dict[str, Any]]:
        """Process a single attachment from ChatGPT export."""
        try:
            # Extract attachment info
            filename = attachment.get("name", "unknown")
            file_type = attachment.get("type", "unknown")
            file_path = attachment.get("path", "")
            
            # Find the actual file in the extracted ZIP
            full_path = os.path.join(temp_dir, file_path)
            if not os.path.exists(full_path):
                return None
            
            # Get file info
            file_size = os.path.getsize(full_path)
            
            # Generate content hash
            with open(full_path, "rb") as f:
                content_hash = hashlib.md5(f.read()).hexdigest()
            
            # Determine MIME type
            mime_type, _ = mimetypes.guess_type(filename)
            if not mime_type:
                mime_type = "application/octet-stream"
            
            # Process text files for embedding
            content_text = ""
            embedded = False
            if mime_type.startswith("text/") or filename.endswith(('.txt', '.md', '.json', '.csv', '.log')):
                try:
                    with open(full_path, "r", encoding="utf-8") as f:
                        content_text = f.read()
                    embedded = True
                except UnicodeDecodeError:
                    # Try with different encoding
                    try:
                        with open(full_path, "r", encoding="latin-1") as f:
                            content_text = f.read()
                        embedded = True
                    except Exception:
                        content_text = f"[Binary or unsupported text file: {filename}]"
            
            # Create attachment data
            attachment_data = {
                "filename": filename,
                "file_type": mime_type,
                "file_path": file_path,
                "file_size": file_size,
                "content_hash": content_hash,
                "content_text": content_text,
                "embedded": embedded
            }
            
            # Store in memory
            await self.memory.store_attachment(message_id, attachment_data)
            
            return attachment_data
            
        except Exception as e:
            print(f"Error processing attachment {filename}: {e}")
            return None
        
    async def update_config(self, new_config: Dict[str, Any]):
        """Update brain configuration."""
        backend_changed = False
        
        # Update config fields
        for key, value in new_config.items():
            if hasattr(self.config, key):
                setattr(self.config, key, value)
            elif hasattr(self.config.llm, key):
                setattr(self.config.llm, key, value)
            elif hasattr(self.config.memory, key):
                setattr(self.config.memory, key, value)
            elif key == "memory" and isinstance(value, dict):
                # Handle nested memory config updates
                for mem_key, mem_value in value.items():
                    if hasattr(self.config.memory, mem_key):
                        old_value = getattr(self.config.memory, mem_key)
                        setattr(self.config.memory, mem_key, mem_value)
                        # Check if backend changed
                        if mem_key == "backend" and old_value != mem_value:
                            backend_changed = True
                            print(f"Backend changed from {old_value} to {mem_value}")
                
        # Recreate memory manager if backend changed
        if backend_changed:
            print(f"Recreating memory manager with backend: {self.config.memory.backend}")
            if self.config.memory.backend == "txtai":
                try:
                    self.memory = TxtaiMemoryManager(self.config)
                    print(f"Switched to txtai memory backend")
                except ImportError:
                    print("txtai not available, falling back to sentence-transformers backend")
                    self.config.memory.backend = "sentence_transformers"
                    self.memory = MemoryManager(self.config)
            else:
                self.memory = MemoryManager(self.config)
                print(f"Switched to sentence-transformers memory backend")
                
        # Reinitialize if needed
        if self._initialized:
            await self.initialize()
            
    async def health_check(self) -> Dict[str, Any]:
        """Perform health check on brain components."""
        health = {
            "status": "healthy",
            "initialized": self._initialized,
            "memory": "ok",
            "llm": "ok",
            "memory_backend": self.config.memory.backend,
            "errors": []
        }
        
        try:
            # Test memory
            if not self._initialized:
                await self.initialize()
        except Exception as e:
            health["memory"] = "error"
            health["errors"].append(f"Memory error: {e}")
            health["status"] = "unhealthy"
            
        try:
            # Test LLM (basic connectivity)
            if not self.config.llm.openai_api_key and not self.config.llm.gemini_api_key:
                health["llm"] = "no_keys"
                health["errors"].append("No LLM API keys configured")
        except Exception as e:
            health["llm"] = "error"
            health["errors"].append(f"LLM error: {e}")
            health["status"] = "unhealthy"
            
        return health 