"""
Command-line interface for Ghost Brain.
"""

import argparse
import sys
import os
from typing import Optional
from pathlib import Path

from .config import BrainConfig
from .server import main as server_main


def create_parser() -> argparse.ArgumentParser:
    """Create the command-line argument parser."""
    parser = argparse.ArgumentParser(
        prog="ghost-brain",
        description="Ghost Brain - Intelligent AI brain with memory management and RAG capabilities",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # Start server with default settings
  ghost-brain server

  # Start server on custom port
  ghost-brain server --port 9000

  # Start server with debug logging
  ghost-brain server --host 0.0.0.0 --port 8000 --log-level debug

  # Start server with custom database
  ghost-brain server --db-path ./data/memory.db --memory-backend txtai

  # Show configuration
  ghost-brain config

  # Show environment variables documentation
  ghost-brain env-docs
        """
    )
    
    subparsers = parser.add_subparsers(dest="command", help="Available commands")
    
    # Server command
    server_parser = subparsers.add_parser("server", help="Start the brain server")
    server_parser.add_argument(
        "--host",
        type=str,
        help="Server host (default: 127.0.0.1)"
    )
    server_parser.add_argument(
        "--port",
        type=int,
        help="Server port (default: 8000)"
    )
    server_parser.add_argument(
        "--log-level",
        choices=["debug", "info", "warning", "error"],
        help="Log level (default: info)"
    )
    server_parser.add_argument(
        "--reload",
        action="store_true",
        help="Enable auto-reload for development"
    )
    
    # Database/Memory options
    server_parser.add_argument(
        "--db-path",
        type=str,
        help="Database file path (default: memory.db)"
    )
    server_parser.add_argument(
        "--memory-backend",
        choices=["sentence_transformers", "txtai"],
        help="Memory backend (default: sentence_transformers)"
    )
    server_parser.add_argument(
        "--embedding-model",
        type=str,
        help="Embedding model (default: sentence-transformers/all-MiniLM-L6-v2)"
    )
    server_parser.add_argument(
        "--max-chunk-tokens",
        type=int,
        help="Maximum tokens per chunk (default: 1500)"
    )
    server_parser.add_argument(
        "--retrieval-k",
        type=int,
        help="Number of chunks to retrieve (default: 3)"
    )
    
    # LLM options
    server_parser.add_argument(
        "--openai-model",
        type=str,
        help="OpenAI model name (default: gpt-4o)"
    )
    server_parser.add_argument(
        "--gemini-model",
        type=str,
        help="Gemini model name (default: gemini-1.5-pro-latest)"
    )
    server_parser.add_argument(
        "--max-tokens",
        type=int,
        help="Maximum tokens for responses (default: 1500)"
    )
    server_parser.add_argument(
        "--temperature",
        type=float,
        help="Response temperature (default: 0.8)"
    )
    
    # General options
    server_parser.add_argument(
        "--system-prompt",
        type=str,
        help="Default system prompt"
    )
    server_parser.add_argument(
        "--user-id",
        type=str,
        help="Default user ID (default: default_user)"
    )
    
    # Config command
    config_parser = subparsers.add_parser("config", help="Show current configuration")
    config_parser.add_argument(
        "--format",
        choices=["text", "json", "env"],
        default="text",
        help="Output format (default: text)"
    )
    
    # Environment variables documentation command
    subparsers.add_parser("env-docs", help="Show environment variables documentation")
    
    # Version command
    subparsers.add_parser("version", help="Show version information")
    
    return parser


def apply_cli_args_to_config(config: BrainConfig, args: argparse.Namespace) -> BrainConfig:
    """Apply command-line arguments to configuration, overriding environment variables."""
    
    # Server configuration
    if args.host:
        config.server.host = args.host
    if args.port:
        config.server.port = args.port
    if args.log_level:
        config.server.log_level = args.log_level
    if args.reload:
        config.server.reload = True
    
    # Memory configuration
    if args.db_path:
        config.memory.db_path = args.db_path
    if args.memory_backend:
        config.memory.backend = args.memory_backend
    if args.embedding_model:
        config.memory.embedding_model = args.embedding_model
    if args.max_chunk_tokens:
        config.memory.max_chunk_tokens = args.max_chunk_tokens
    if args.retrieval_k:
        config.memory.retrieval_k = args.retrieval_k
    
    # LLM configuration
    if args.openai_model:
        config.llm.openai_model = args.openai_model
    if args.gemini_model:
        config.llm.gemini_model = args.gemini_model
    if args.max_tokens:
        config.llm.max_tokens = args.max_tokens
    if args.temperature:
        config.llm.temperature = args.temperature
    
    # General configuration
    if args.system_prompt:
        config.system_prompt = args.system_prompt
    if args.user_id:
        config.user_id = args.user_id
    
    return config


def show_config(config: BrainConfig, format_type: str = "text"):
    """Display configuration in the specified format."""
    
    if format_type == "json":
        import json
        config_dict = {
            "server": {
                "host": config.server.host,
                "port": config.server.port,
                "log_level": config.server.log_level,
                "reload": config.server.reload
            },
            "memory": {
                "db_path": config.memory.db_path,
                "embedding_model": config.memory.embedding_model,
                "max_chunk_tokens": config.memory.max_chunk_tokens,
                "retrieval_k": config.memory.retrieval_k,
                "backend": config.memory.backend
            },
            "llm": {
                "openai_model": config.llm.openai_model,
                "gemini_model": config.llm.gemini_model,
                "max_tokens": config.llm.max_tokens,
                "temperature": config.llm.temperature,
                "openai_api_key": "***" if config.llm.openai_api_key else None,
                "gemini_api_key": "***" if config.llm.gemini_api_key else None
            },
            "general": {
                "system_prompt": config.system_prompt,
                "user_id": config.user_id
            }
        }
        print(json.dumps(config_dict, indent=2))
    
    elif format_type == "env":
        print(f"BRAIN_SERVER_HOST={config.server.host}")
        print(f"BRAIN_SERVER_PORT={config.server.port}")
        print(f"BRAIN_LOG_LEVEL={config.server.log_level}")
        print(f"BRAIN_RELOAD={str(config.server.reload).lower()}")
        print(f"BRAIN_DB_PATH={config.memory.db_path}")
        print(f"BRAIN_MEMORY_BACKEND={config.memory.backend}")
        print(f"BRAIN_EMBEDDING_MODEL={config.memory.embedding_model}")
        print(f"BRAIN_MAX_CHUNK_TOKENS={config.memory.max_chunk_tokens}")
        print(f"BRAIN_RETRIEVAL_K={config.memory.retrieval_k}")
        print(f"BRAIN_OPENAI_MODEL={config.llm.openai_model}")
        print(f"BRAIN_GEMINI_MODEL={config.llm.gemini_model}")
        print(f"BRAIN_MAX_TOKENS={config.llm.max_tokens}")
        print(f"BRAIN_TEMPERATURE={config.llm.temperature}")
        print(f"BRAIN_SYSTEM_PROMPT={config.system_prompt}")
        print(f"BRAIN_USER_ID={config.user_id}")
    
    else:  # text format
        print("🧠 Ghost Brain Configuration")
        print("=" * 50)
        
        print("\n📡 Server Configuration:")
        print(f"  Host: {config.server.host}")
        print(f"  Port: {config.server.port}")
        print(f"  Log Level: {config.server.log_level}")
        print(f"  Auto-reload: {config.server.reload}")
        
        print("\n💾 Memory Configuration:")
        print(f"  Database: {config.memory.db_path}")
        print(f"  Backend: {config.memory.backend}")
        print(f"  Embedding Model: {config.memory.embedding_model}")
        print(f"  Max Chunk Tokens: {config.memory.max_chunk_tokens}")
        print(f"  Retrieval K: {config.memory.retrieval_k}")
        
        print("\n🤖 LLM Configuration:")
        print(f"  OpenAI Model: {config.llm.openai_model}")
        print(f"  Gemini Model: {config.llm.gemini_model}")
        print(f"  Max Tokens: {config.llm.max_tokens}")
        print(f"  Temperature: {config.llm.temperature}")
        print(f"  OpenAI API Key: {'✓ Set' if config.llm.openai_api_key else '✗ Not set'}")
        print(f"  Gemini API Key: {'✓ Set' if config.llm.gemini_api_key else '✗ Not set'}")
        
        print("\n⚙️ General Configuration:")
        print(f"  System Prompt: {config.system_prompt}")
        print(f"  User ID: {config.user_id}")


def show_version():
    """Show version information."""
    try:
        from . import __version__
        version = __version__
    except ImportError:
        version = "unknown"
    
    print(f"Ghost Brain v{version}")
    print("Intelligent AI brain with memory management and RAG capabilities")


def main():
    """Main CLI entry point."""
    parser = create_parser()
    args = parser.parse_args()
    
    if not args.command:
        parser.print_help()
        return 1
    
    try:
        if args.command == "server":
            # Load configuration from environment variables
            config = BrainConfig()
            
            # Apply CLI arguments (override environment variables)
            config = apply_cli_args_to_config(config, args)
            
            # Set the configuration in environment for the server
            os.environ["BRAIN_SERVER_HOST"] = config.server.host
            os.environ["BRAIN_SERVER_PORT"] = str(config.server.port)
            os.environ["BRAIN_LOG_LEVEL"] = config.server.log_level
            os.environ["BRAIN_RELOAD"] = str(config.server.reload).lower()
            os.environ["BRAIN_DB_PATH"] = config.memory.db_path
            os.environ["BRAIN_MEMORY_BACKEND"] = config.memory.backend
            os.environ["BRAIN_EMBEDDING_MODEL"] = config.memory.embedding_model
            os.environ["BRAIN_MAX_CHUNK_TOKENS"] = str(config.memory.max_chunk_tokens)
            os.environ["BRAIN_RETRIEVAL_K"] = str(config.memory.retrieval_k)
            os.environ["BRAIN_OPENAI_MODEL"] = config.llm.openai_model
            os.environ["BRAIN_GEMINI_MODEL"] = config.llm.gemini_model
            os.environ["BRAIN_MAX_TOKENS"] = str(config.llm.max_tokens)
            os.environ["BRAIN_TEMPERATURE"] = str(config.llm.temperature)
            os.environ["BRAIN_SYSTEM_PROMPT"] = config.system_prompt
            os.environ["BRAIN_USER_ID"] = config.user_id
            
            # Start the server
            server_main()
            
        elif args.command == "config":
            config = BrainConfig()
            show_config(config, args.format)
            
        elif args.command == "env-docs":
            config = BrainConfig()
            print(config.get_environment_variables_doc())
            
        elif args.command == "version":
            show_version()
            
    except KeyboardInterrupt:
        print("\n👋 Goodbye!")
        return 0
    except Exception as e:
        print(f"❌ Error: {e}", file=sys.stderr)
        return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main()) 