"""
Configuration management for Ghost Brain.
"""

import os
from typing import Dict, Any, Optional, Literal
from pydantic import BaseModel, Field


class LLMConfig(BaseModel):
    """Configuration for LLM providers."""
    openai_api_key: Optional[str] = None
    gemini_api_key: Optional[str] = None
    openai_model: str = "gpt-4o"
    gemini_model: str = "gemini-1.5-pro-latest"
    summarizer_model: str = "gemini-1.5-flash-latest"
    summarizer_type: str = "gemini"
    max_tokens: int = 1500
    temperature: float = 0.8


class MemoryConfig(BaseModel):
    """Configuration for memory management."""
    db_path: str = "memory.db"
    embedding_model: str = "sentence-transformers/all-MiniLM-L6-v2"
    max_chunk_tokens: int = 1500
    retrieval_k: int = 3
    backend: Literal["sentence_transformers", "txtai"] = "sentence_transformers"


class ServerConfig(BaseModel):
    """Configuration for the brain server."""
    host: str = "127.0.0.1"
    port: int = 8000
    log_level: str = "info"
    reload: bool = False


class BrainConfig(BaseModel):
    """Main configuration for Ghost Brain."""
    llm: LLMConfig = LLMConfig()
    memory: MemoryConfig = MemoryConfig()
    server: ServerConfig = ServerConfig()
    system_prompt: str = "You are a helpful AI assistant."
    user_id: str = "default_user"
    
    def __init__(self, **data):
        super().__init__(**data)
        self._load_environment_variables()
    
    def _load_environment_variables(self):
        """Load configuration from environment variables."""
        
        # LLM Configuration
        if not self.llm.openai_api_key:
            self.llm.openai_api_key = os.getenv("OPENAI_API_KEY")
        if not self.llm.gemini_api_key:
            self.llm.gemini_api_key = os.getenv("GEMINI_API_KEY")
        
        # LLM Model Settings
        if os.getenv("BRAIN_OPENAI_MODEL"):
            self.llm.openai_model = os.getenv("BRAIN_OPENAI_MODEL")
        if os.getenv("BRAIN_GEMINI_MODEL"):
            self.llm.gemini_model = os.getenv("BRAIN_GEMINI_MODEL")
        if os.getenv("BRAIN_SUMMARIZER_MODEL"):
            self.llm.summarizer_model = os.getenv("BRAIN_SUMMARIZER_MODEL")
        if os.getenv("BRAIN_SUMMARIZER_TYPE"):
            self.llm.summarizer_type = os.getenv("BRAIN_SUMMARIZER_TYPE")
        
        # Handle numeric values with error checking
        if os.getenv("BRAIN_MAX_TOKENS"):
            try:
                self.llm.max_tokens = int(os.getenv("BRAIN_MAX_TOKENS"))
            except (ValueError, TypeError):
                pass  # Keep default value
        if os.getenv("BRAIN_TEMPERATURE"):
            try:
                self.llm.temperature = float(os.getenv("BRAIN_TEMPERATURE"))
            except (ValueError, TypeError):
                pass  # Keep default value
        
        # Memory Configuration
        if os.getenv("BRAIN_DB_PATH"):
            self.memory.db_path = os.getenv("BRAIN_DB_PATH")
        if os.getenv("BRAIN_EMBEDDING_MODEL"):
            self.memory.embedding_model = os.getenv("BRAIN_EMBEDDING_MODEL")
        
        # Handle numeric values with error checking
        if os.getenv("BRAIN_MAX_CHUNK_TOKENS"):
            try:
                self.memory.max_chunk_tokens = int(os.getenv("BRAIN_MAX_CHUNK_TOKENS"))
            except (ValueError, TypeError):
                pass  # Keep default value
        if os.getenv("BRAIN_RETRIEVAL_K"):
            try:
                self.memory.retrieval_k = int(os.getenv("BRAIN_RETRIEVAL_K"))
            except (ValueError, TypeError):
                pass  # Keep default value
        
        if os.getenv("BRAIN_MEMORY_BACKEND"):
            backend = os.getenv("BRAIN_MEMORY_BACKEND")
            if backend in ["sentence_transformers", "txtai"]:
                self.memory.backend = backend
        
        # Server Configuration
        if os.getenv("BRAIN_SERVER_HOST"):
            self.server.host = os.getenv("BRAIN_SERVER_HOST")
        
        # Handle numeric values with error checking
        if os.getenv("BRAIN_SERVER_PORT"):
            try:
                self.server.port = int(os.getenv("BRAIN_SERVER_PORT"))
            except (ValueError, TypeError):
                pass  # Keep default value
        
        if os.getenv("BRAIN_LOG_LEVEL"):
            self.server.log_level = os.getenv("BRAIN_LOG_LEVEL")
        if os.getenv("BRAIN_RELOAD"):
            self.server.reload = os.getenv("BRAIN_RELOAD").lower() in ["true", "1", "yes"]
        
        # General Configuration
        if os.getenv("BRAIN_SYSTEM_PROMPT"):
            self.system_prompt = os.getenv("BRAIN_SYSTEM_PROMPT")
        if os.getenv("BRAIN_USER_ID"):
            self.user_id = os.getenv("BRAIN_USER_ID")
    
    def get_environment_variables_doc(self) -> str:
        """Generate documentation for all supported environment variables."""
        return """
# Ghost Brain Environment Variables

## LLM Configuration
OPENAI_API_KEY              # OpenAI API key
GEMINI_API_KEY              # Google Gemini API key
BRAIN_OPENAI_MODEL          # OpenAI model name (default: gpt-4o)
BRAIN_GEMINI_MODEL          # Gemini model name (default: gemini-1.5-pro-latest)
BRAIN_SUMMARIZER_MODEL      # Summarizer model (default: gemini-1.5-flash-latest)
BRAIN_SUMMARIZER_TYPE       # Summarizer type (default: gemini)
BRAIN_MAX_TOKENS            # Maximum tokens for responses (default: 1500)
BRAIN_TEMPERATURE           # Response temperature (default: 0.8)

## Memory Configuration
BRAIN_DB_PATH               # Database file path (default: memory.db)
BRAIN_EMBEDDING_MODEL       # Embedding model (default: sentence-transformers/all-MiniLM-L6-v2)
BRAIN_MAX_CHUNK_TOKENS      # Maximum tokens per chunk (default: 1500)
BRAIN_RETRIEVAL_K           # Number of chunks to retrieve (default: 3)
BRAIN_MEMORY_BACKEND        # Memory backend: sentence_transformers or txtai (default: sentence_transformers)

## Server Configuration
BRAIN_SERVER_HOST           # Server host (default: 127.0.0.1)
BRAIN_SERVER_PORT           # Server port (default: 8000)
BRAIN_LOG_LEVEL             # Log level: debug, info, warning, error (default: info)
BRAIN_RELOAD                # Enable auto-reload (default: false)

## General Configuration
BRAIN_SYSTEM_PROMPT         # Default system prompt (default: "You are a helpful AI assistant.")
BRAIN_USER_ID               # Default user ID (default: default_user)

## Example .env file:
OPENAI_API_KEY=sk-your-openai-key
GEMINI_API_KEY=your-gemini-key
BRAIN_SERVER_PORT=8000
BRAIN_SERVER_HOST=0.0.0.0
BRAIN_DB_PATH=./data/memory.db
BRAIN_MEMORY_BACKEND=txtai
BRAIN_LOG_LEVEL=info
"""


# Default configuration
DEFAULT_CONFIG = BrainConfig() 