"""
txtai-based memory management for Ghost Brain.
Provides the same interface as MemoryManager but uses txtai for enhanced RAG capabilities.
"""

import datetime
import asyncio
import json
import sqlite3
from collections import deque
from typing import List, Dict, Any, Optional, Tuple
from pathlib import Path

try:
    from txtai.embeddings import Embeddings
    from txtai.pipeline import Pipeline
    TXTAI_AVAILABLE = True
except ImportError:
    TXTAI_AVAILABLE = False
    print("Warning: txtai not available. Install with: pip install txtai")

from .config import BrainConfig


class TxtaiMemoryManager:
    """txtai-based memory manager with enhanced RAG capabilities."""
    
    def __init__(self, config: BrainConfig):
        self.config = config
        self._embeddings = None
        self._pipeline = None
        self._txtai_index_path = Path(config.memory.db_path).parent / "txtai_index"
        
    def _initialize_txtai(self):
        """Initialize txtai embeddings and pipeline."""
        if not TXTAI_AVAILABLE:
            raise ImportError("txtai not available. Install with: pip install txtai")
            
        if self._embeddings is None:
            # Configure txtai embeddings - use a simpler model for txtai
            embeddings_config = {
                "path": "sentence-transformers/all-MiniLM-L6-v2",  # Use full HF path
                "content": True,
                "backend": "faiss",
                "index": str(self._txtai_index_path)
            }
            
            self._embeddings = Embeddings(embeddings_config)
            print(f"Initialized txtai embeddings with model: {embeddings_config['path']}")
            
        # Note: Pipeline is not needed for basic functionality, removed for now

    def initialize_database(self):
        """Initialize the memory database with required tables."""
        conn = sqlite3.connect(self.config.memory.db_path)
        cursor = conn.cursor()

        # Messages table - stores raw conversation data
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS messages (
                unique_key TEXT PRIMARY KEY,
                message_id TEXT,
                author_id TEXT,
                timestamp TEXT,
                content TEXT,
                user_id TEXT,
                edited_from_id TEXT
            )
        """)

        # txtai_chunks table - stores txtai-indexed conversation segments
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS txtai_chunks (
                chunk_id TEXT PRIMARY KEY,
                user_id TEXT,
                content TEXT,
                metadata TEXT,
                timestamp TEXT,
                created_at TEXT
            )
        """)

        # Attachments table - stores metadata about files attached to messages
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS attachments (
                attachment_id TEXT PRIMARY KEY,
                message_id TEXT,
                filename TEXT,
                file_type TEXT,
                file_path TEXT,
                file_size INTEGER,
                content_hash TEXT,
                content_text TEXT,
                embedded BOOLEAN DEFAULT FALSE,
                created_at TEXT,
                FOREIGN KEY (message_id) REFERENCES messages (unique_key)
            )
        """)

        conn.commit()
        conn.close()
        print(f"Initialized txtai memory database: {self.config.memory.db_path}")
        
        # Initialize txtai components
        self._initialize_txtai()

    async def search_similar_chunks(self, query_text: str, top_k: int = 3) -> List[str]:
        """Search for similar conversation chunks using txtai with citations."""
        if not TXTAI_AVAILABLE:
            print("Warning: txtai not available, returning empty results")
            return []
            
        try:
            # Get documents from database
            def _fetch_documents():
                conn = sqlite3.connect(self.config.memory.db_path)
                cursor = conn.cursor()
                cursor.execute(
                    "SELECT chunk_id, content, metadata FROM txtai_chunks WHERE user_id=?",
                    (self.config.user_id,)
                )
                rows = cursor.fetchall()
                conn.close()
                return rows

            rows = await asyncio.to_thread(_fetch_documents)
            if not rows:
                return []

            # Prepare documents for txtai
            documents = []
            for chunk_id, content, metadata in rows:
                meta = json.loads(metadata) if metadata else {}
                documents.append({
                    "id": chunk_id,
                    "text": content,
                    "metadata": meta
                })

            # Index documents in txtai
            if documents:
                await asyncio.to_thread(self._embeddings.index, documents)

            # Search with txtai
            results = await asyncio.to_thread(
                self._embeddings.search, 
                query_text, 
                top_k
            )

            # Format results with citations
            similar_chunks = []
            for result in results:
                chunk_text = result["text"]
                metadata = result.get("metadata", {})
                
                # Add citation if available
                if metadata.get("timestamp"):
                    citation = f"[{metadata['timestamp']}] "
                    chunk_text = citation + chunk_text
                    
                similar_chunks.append(chunk_text)

            return similar_chunks

        except Exception as e:
            print(f"Error in txtai search: {e}")
            return []

    async def get_chat_history(self, current_message: str, limit: int = 10) -> List[Dict[str, Any]]:
        """Get recent chat history combined with relevant memory using txtai."""
        history_messages = []

        # Retrieve additional context from txtai memory
        try:
            retrieved = await self.search_similar_chunks(
                current_message,
                top_k=self.config.memory.retrieval_k
            )
            
            for chunk in retrieved[::-1]:
                history_messages.insert(
                    0,
                    {
                        "role": "system",
                        "content": chunk,
                        "author_id": "txtai_memory",
                        "timestamp": "0",
                        "message_id": "0",
                    }
                )
        except Exception as e:
            print(f"Error retrieving similar chunks: {e}")

        return history_messages

    async def store_messages(self, messages: List[Dict[str, Any]]):
        """Store messages in the database."""
        def _store():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            for msg in messages:
                unique_key = f"{self.config.user_id}_{msg['message_id']}_{msg['timestamp']}"
                cursor.execute(
                    "INSERT OR REPLACE INTO messages (unique_key, message_id, author_id, timestamp, content, user_id) VALUES (?, ?, ?, ?, ?, ?)",
                    (unique_key, msg['message_id'], msg['author_id'], msg['timestamp'], msg['content'], self.config.user_id)
                )
            
            conn.commit()
            conn.close()

        await asyncio.to_thread(_store)

    async def store_chunks(self, chunks: List[Dict[str, Any]], reasoning_text: str):
        """Store conversation chunks in txtai format."""
        def _store():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            for i, chunk in enumerate(chunks):
                chunk_id = f"{self.config.user_id}_chunk_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}_{i}"
                
                # Prepare metadata
                metadata = {
                    "user_id": self.config.user_id,
                    "timestamp": datetime.datetime.now().isoformat(),
                    "reasoning": reasoning_text,
                    "chunk_index": i,
                    "total_chunks": len(chunks)
                }
                
                cursor.execute(
                    "INSERT OR REPLACE INTO txtai_chunks (chunk_id, user_id, content, metadata, timestamp, created_at) VALUES (?, ?, ?, ?, ?, ?)",
                    (
                        chunk_id,
                        self.config.user_id,
                        chunk["content"],
                        json.dumps(metadata),
                        metadata["timestamp"],
                        metadata["timestamp"]
                    )
                )
            
            conn.commit()
            conn.close()

        await asyncio.to_thread(_store)

    async def chunk_conversation(self, conversation_history: List[Dict[str, Any]], max_chunk_tokens: int) -> List[Dict[str, Any]]:
        """Chunk conversation into segments for txtai indexing."""
        chunks = []
        current_chunk = {"content": "", "messages": []}
        current_tokens = 0
        
        for msg in conversation_history:
            content = msg.get("content", "")
            
            # Add attachment content to message if available
            if 'attachments' in msg and msg['attachments']:
                attachment_texts = []
                for attachment in msg['attachments']:
                    if attachment.get('embedded') and attachment.get('content_text'):
                        attachment_texts.append(f"[Attachment: {attachment['filename']}]\n{attachment['content_text']}")
                
                if attachment_texts:
                    content += "\n\n" + "\n\n".join(attachment_texts)
            
            # Simple token estimation (words * 1.3)
            estimated_tokens = len(content.split()) * 1.3
            
            if current_tokens + estimated_tokens > max_chunk_tokens and current_chunk["content"]:
                chunks.append(current_chunk)
                current_chunk = {"content": "", "messages": []}
                current_tokens = 0
            
            current_chunk["content"] += f"{msg.get('author_id', 'unknown')}: {content}\n"
            current_chunk["messages"].append(msg)
            current_tokens += estimated_tokens
        
        if current_chunk["content"]:
            chunks.append(current_chunk)
        
        return chunks

    async def process_and_store_memory(self, history: List[Dict[str, Any]], response: str, reasoning: str):
        """Process conversation history and store in txtai memory."""
        try:
            # Store raw messages
            await self.store_messages(history)
            
            # Add response to history for chunking
            full_history = history + [{
                "role": "assistant",
                "content": response,
                "message_id": "response",
                "timestamp": datetime.datetime.now().isoformat(),
                "author_id": "assistant"
            }]
            
            # Chunk conversation
            chunks = await self.chunk_conversation(full_history, self.config.memory.max_chunk_tokens)
            
            # Store chunks in txtai format
            await self.store_chunks(chunks, reasoning)
            
            print(f"Stored {len(chunks)} chunks in txtai memory")
            
        except Exception as e:
            print(f"Error processing and storing memory: {e}")

    async def get_memory_stats(self) -> Dict[str, Any]:
        """Get memory statistics."""
        def _get_stats():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            # Count messages
            cursor.execute("SELECT COUNT(*) FROM messages WHERE user_id=?", (self.config.user_id,))
            message_count = cursor.fetchone()[0]
            
            # Count chunks
            cursor.execute("SELECT COUNT(*) FROM txtai_chunks WHERE user_id=?", (self.config.user_id,))
            chunk_count = cursor.fetchone()[0]
            
            # Count attachments
            cursor.execute("SELECT COUNT(*) FROM attachments")
            attachment_count = cursor.fetchone()[0]
            
            # Count embedded attachments
            cursor.execute("SELECT COUNT(*) FROM attachments WHERE embedded = 1")
            embedded_attachment_count = cursor.fetchone()[0]
            
            conn.close()
            return message_count, chunk_count, attachment_count, embedded_attachment_count

        message_count, chunk_count, attachment_count, embedded_attachment_count = await asyncio.to_thread(_get_stats)
        
        return {
            "user_id": self.config.user_id,
            "db_path": self.config.memory.db_path,
            "embedding_model": self.config.memory.embedding_model,
            "message_count": message_count,
            "chunk_count": chunk_count,
            "attachment_count": attachment_count,
            "embedded_attachment_count": embedded_attachment_count,
            "backend": "txtai",
            "txtai_available": TXTAI_AVAILABLE
        }

    async def store_attachment(self, message_id: str, attachment_data: Dict[str, Any]):
        """Store attachment metadata in the database."""
        def _store():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            attachment_id = f"{self.config.user_id}_{message_id}_{attachment_data['filename']}"
            
            cursor.execute(
                """INSERT OR REPLACE INTO attachments 
                   (attachment_id, message_id, filename, file_type, file_path, file_size, content_hash, content_text, embedded, created_at) 
                   VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                (
                    attachment_id,
                    message_id,
                    attachment_data['filename'],
                    attachment_data.get('file_type', ''),
                    attachment_data.get('file_path', ''),
                    attachment_data.get('file_size', 0),
                    attachment_data.get('content_hash', ''),
                    attachment_data.get('content_text', ''),
                    attachment_data.get('embedded', False),
                    datetime.datetime.now().isoformat()
                )
            )
            
            conn.commit()
            conn.close()

        await asyncio.to_thread(_store)

    async def get_attachments_for_message(self, message_id: str) -> List[Dict[str, Any]]:
        """Get all attachments for a specific message."""
        def _get():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            cursor.execute(
                "SELECT filename, file_type, file_path, file_size, content_hash, content_text, embedded FROM attachments WHERE message_id = ?",
                (message_id,)
            )
            
            rows = cursor.fetchall()
            conn.close()
            
            attachments = []
            for row in rows:
                attachments.append({
                    'filename': row[0],
                    'file_type': row[1],
                    'file_path': row[2],
                    'file_size': row[3],
                    'content_hash': row[4],
                    'content_text': row[5],
                    'embedded': bool(row[6])
                })
            
            return attachments

        return await asyncio.to_thread(_get)

    async def get_attachment_stats(self) -> Dict[str, Any]:
        """Get attachment statistics."""
        def _get_stats():
            conn = sqlite3.connect(self.config.memory.db_path)
            cursor = conn.cursor()
            
            # Count total attachments
            cursor.execute("SELECT COUNT(*) FROM attachments")
            total_attachments = cursor.fetchone()[0]
            
            # Count embedded attachments
            cursor.execute("SELECT COUNT(*) FROM attachments WHERE embedded = 1")
            embedded_attachments = cursor.fetchone()[0]
            
            # Count by file type
            cursor.execute("SELECT file_type, COUNT(*) FROM attachments GROUP BY file_type")
            type_counts = dict(cursor.fetchall())
            
            conn.close()
            return total_attachments, embedded_attachments, type_counts

        total_attachments, embedded_attachments, type_counts = await asyncio.to_thread(_get_stats)
        
        return {
            "total_attachments": total_attachments,
            "embedded_attachments": embedded_attachments,
            "type_counts": type_counts
        } 