"""
FastAPI server for Ghost Brain.
"""

import asyncio
from typing import Dict, Any, List, Optional
from fastapi import FastAPI, Depends, HTTPException, UploadFile, File, Form
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel
import uvicorn

from .brain import Brain
from .config import BrainConfig


# Global brain instance
_brain_instance: Optional[Brain] = None


class ChatRequest(BaseModel):
    message: str
    history: Optional[List[Dict[str, Any]]] = None
    user_id: Optional[str] = None
    system_prompt: Optional[str] = None
    model_type: Optional[str] = None
    model_name: Optional[str] = None
    use_structured_response: bool = True


class ChatResponse(BaseModel):
    response: str
    reasoning: Optional[str] = None
    memory_chunks: int
    timestamp: str
    model_type: str
    model_name: str


class SearchRequest(BaseModel):
    query: str
    top_k: int = 3


class ConfigUpdateRequest(BaseModel):
    config: Dict[str, Any]


class ChatHistoryEmbedRequest(BaseModel):
    chat_history_path: str
    user_id: Optional[str] = None


class ChatHistoryEmbedResponse(BaseModel):
    status: str
    files_processed: int
    chunks_embedded: int
    chunks_skipped: int
    errors: List[str]


class HealthResponse(BaseModel):
    status: str
    initialized: bool
    memory: str
    llm: str
    errors: List[str]


class ChatHistoryImportZipResponse(BaseModel):
    status: str
    files_processed: int
    chunks_embedded: int
    chunks_skipped: int
    attachments_processed: int
    errors: List[str]


class AttachmentInfo(BaseModel):
    filename: str
    file_type: str
    file_path: str
    file_size: int
    content_hash: str
    content_text: str
    embedded: bool


class AttachmentStats(BaseModel):
    total_attachments: int
    embedded_attachments: int
    type_counts: Dict[str, int]


async def get_brain():
    """Dependency to get brain instance."""
    global _brain_instance
    if _brain_instance is None:
        config = BrainConfig()
        _brain_instance = Brain(config)
        await _brain_instance.initialize()
    return _brain_instance


def create_app() -> FastAPI:
    """Create and configure FastAPI application."""
    app = FastAPI(
        title="Ghost Brain API",
        description="Intelligent AI brain with memory management and RAG capabilities",
        version="0.0.1"
    )
    
    # Add CORS middleware
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )
    
    @app.get("/health", response_model=HealthResponse)
    async def health_check(brain: Brain = Depends(get_brain)):
        return await brain.health_check()
    
    @app.post("/chat", response_model=ChatResponse)
    async def chat(request: ChatRequest, brain: Brain = Depends(get_brain)):
        try:
            result = await brain.process_message(
                message=request.message,
                history=request.history,
                user_id=request.user_id,
                system_prompt=request.system_prompt,
                model_type=request.model_type,
                model_name=request.model_name,
                use_structured_response=request.use_structured_response
            )
            return ChatResponse(**result)
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Processing error: {str(e)}")
    
    @app.post("/memory/search")
    async def search_memory(request: SearchRequest, brain: Brain = Depends(get_brain)):
        try:
            chunks = await brain.search_memory(request.query, request.top_k)
            return {"chunks": chunks, "query": request.query, "top_k": request.top_k}
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Search error: {str(e)}")
    
    @app.get("/memory/stats")
    async def get_memory_stats(brain: Brain = Depends(get_brain)):
        try:
            return await brain.get_memory_stats()
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Stats error: {str(e)}")
    
    @app.post("/config/update")
    async def update_config(request: ConfigUpdateRequest, brain: Brain = Depends(get_brain)):
        try:
            await brain.update_config(request.config)
            return {"status": "updated", "config": request.config}
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Config update error: {str(e)}")
    
    @app.post("/chat-history/embed", response_model=ChatHistoryEmbedResponse)
    async def embed_chat_history(request: ChatHistoryEmbedRequest, brain: Brain = Depends(get_brain)):
        try:
            result = await brain.embed_chat_history(
                chat_history_path=request.chat_history_path,
                user_id=request.user_id
            )
            return ChatHistoryEmbedResponse(**result)
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Chat history embedding error: {str(e)}")
    
    @app.post("/chat-history/import-zip", response_model=ChatHistoryImportZipResponse)
    async def import_chatgpt_zip(
        file: UploadFile = File(...),
        user_id: Optional[str] = Form(None),
        brain: Brain = Depends(get_brain)
    ):
        try:
            result = await brain.import_chatgpt_zip(file, user_id)
            return ChatHistoryImportZipResponse(**result)
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"ChatGPT ZIP import error: {str(e)}")
    
    @app.get("/attachments/message/{message_id}", response_model=List[AttachmentInfo])
    async def get_message_attachments(message_id: str, brain: Brain = Depends(get_brain)):
        try:
            attachments = await brain.memory.get_attachments_for_message(message_id)
            return [AttachmentInfo(**attachment) for attachment in attachments]
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Attachment retrieval error: {str(e)}")
    
    @app.get("/attachments/stats", response_model=AttachmentStats)
    async def get_attachment_stats(brain: Brain = Depends(get_brain)):
        try:
            stats = await brain.memory.get_attachment_stats()
            return AttachmentStats(**stats)
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"Attachment stats error: {str(e)}")
    
    @app.get("/config/environment-variables")
    async def get_environment_variables_doc():
        """Get documentation for all supported environment variables."""
        config = BrainConfig()
        return {"documentation": config.get_environment_variables_doc()}
    
    @app.get("/")
    async def root():
        return {
            "name": "Ghost Brain API",
            "version": "0.0.1",
            "endpoints": [
                "GET /health - Health check",
                "POST /chat - Process chat message",
                "POST /memory/search - Search memory",
                "GET /memory/stats - Get memory statistics",
                "POST /config/update - Update configuration",
                "GET /config/environment-variables - Get environment variables documentation",
                "POST /chat-history/embed - Embed chat history from files",
                "POST /chat-history/import-zip - Import ChatGPT ZIP with attachments",
                "GET /attachments/message/{message_id} - Get message attachments",
                "GET /attachments/stats - Get attachment statistics"
            ]
        }
    
    return app


def main():
    """Main entry point for the brain server."""
    # Load configuration from environment variables
    config = BrainConfig()
    
    app = create_app()
    
    print("Starting Ghost Brain server...")
    print(f"API will be available at: http://{config.server.host}:{config.server.port}")
    print(f"Health check: http://{config.server.host}:{config.server.port}/health")
    print(f"Environment variables doc: http://{config.server.host}:{config.server.port}/config/environment-variables")
    
    # Print configuration summary
    print("\nConfiguration:")
    print(f"  Server: {config.server.host}:{config.server.port}")
    print(f"  Database: {config.memory.db_path}")
    print(f"  Memory Backend: {config.memory.backend}")
    print(f"  OpenAI Model: {config.llm.openai_model}")
    print(f"  Gemini Model: {config.llm.gemini_model}")
    print(f"  Log Level: {config.server.log_level}")
    
    uvicorn.run(
        app,
        host=config.server.host,
        port=config.server.port,
        log_level=config.server.log_level,
        reload=config.server.reload
    )


if __name__ == "__main__":
    main() 